@@ -209,6 +209,7 @@ ev_view_size_request (GtkWidget      *widget,
 	if (GTK_WIDGET_REALIZED (widget)) {
 		if (view->document) {
 			ev_document_get_page_size (view->document,
+						   -1,
 						   &requisition->width,
 						   &requisition->height);
 		} else {
@@ -411,7 +412,9 @@ expose_bin_window (GtkWidget      *widget,
 			    event->area.x, event->area.y,
 			    event->area.width, event->area.height);
 
-	highlight_find_results (view);
+	if (EV_IS_DOCUMENT_FIND (view->document)) {
+		highlight_find_results (view);
+	}
 
 	if (view->has_selection) {
 		draw_rubberband (widget, view->bin_window,
@@ -941,13 +944,20 @@ update_find_status_message (EvView *view)
 }
 
 static void
-set_document_page (EvView *view, int page)
+set_document_page (EvView *view, int new_page)
 {
+	int page;
+	int pages;
+
+	pages = ev_document_get_n_pages (view->document);
+	page = CLAMP (new_page, 1, pages);
+
 	if (view->document) {
 		int old_page = ev_document_get_page (view->document);
 		int old_width, old_height;
 
 		ev_document_get_page_size (view->document,
+					   -1, 
 					   &old_width, &old_height);
 
 		if (old_page != page) {
@@ -962,6 +972,7 @@ set_document_page (EvView *view, int page)
 
 			view->has_selection = FALSE;
 			ev_document_get_page_size (view->document,
+						   -1, 
 						   &width, &height);
 			if (width != old_width || height != old_height)
 				gtk_widget_queue_resize (GTK_WIDGET (view));
@@ -972,7 +983,10 @@ set_document_page (EvView *view, int page)
 
 		view->find_page = page;
 		view->find_result = 0;
-		update_find_status_message (view);
+
+		if (EV_IS_DOCUMENT_FIND (view->document)) {
+			update_find_status_message (view);
+		}
 	}
 }
 
@@ -1013,11 +1027,17 @@ ensure_rectangle_is_visible (EvView *view, GdkRectangle *rect)
 static void
 jump_to_find_result (EvView *view)
 {
+	EvDocumentFind *find = EV_DOCUMENT_FIND (view->document);
 	GdkRectangle rect;
+	int n_results;
 
-	ev_document_find_get_result (EV_DOCUMENT_FIND (view->document),
-				     view->find_result, &rect);
-	ensure_rectangle_is_visible (view, &rect);
+	n_results = ev_document_find_get_n_results (find);
+
+	if (n_results > view->find_result) {
+		ev_document_find_get_result
+			(find, view->find_result, &rect);
+		ensure_rectangle_is_visible (view, &rect);
+	}
 }
 
 static void
@@ -1085,9 +1105,8 @@ ev_view_set_document (EvView     *view,
 
 	if (document != view->document) {
 		if (view->document) {
-                        g_signal_handlers_disconnect_by_func (view->document,
-                                                              find_changed_cb,
-                                                              view);
+                        g_signal_handlers_disconnect_by_func
+				(view->document, find_changed_cb, view);
 			g_object_unref (view->document);
                 }
 
@@ -1097,11 +1116,12 @@ ev_view_set_document (EvView     *view,
 
 		if (view->document) {
 			g_object_ref (view->document);
-			if (EV_IS_DOCUMENT_FIND (view->document))
+			if (EV_IS_DOCUMENT_FIND (view->document)) {
 				g_signal_connect (view->document,
 						  "find_changed",
 						  G_CALLBACK (find_changed_cb),
 						  view);
+			}
 			g_signal_connect (view->document,
 					  "changed",
 					  G_CALLBACK (document_changed_callback),
@@ -1214,7 +1234,7 @@ ev_view_best_fit (EvView *view)
 	int width, height;
 
 	width = height = 0;
-	ev_document_get_page_size (view->document, &width, &height);
+	ev_document_get_page_size (view->document, -1, &width, &height);
 
 	scale = 1.0;
 	if (width != 0 && height != 0) {
@@ -1236,7 +1256,7 @@ ev_view_fit_width (EvView *view)
 	int width;
 
 	width = 0;
-	ev_document_get_page_size (view->document, &width, NULL);
+	ev_document_get_page_size (view->document, -1, &width, NULL);
 
 	scale = 1.0;
 	if (width != 0)
